/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */

#pragma once

#include <stdio.h>
#include <unistd.h>
#include <string.h>

//Standard Library Includes
#include <iostream>
#include <sstream>
#include <cstdlib>
#include <assert.h>

#include <intel-coi/common/COIResult_common.h>

//Namespaces
using namespace std;

#define XTH  100

//Useful error checking macro
#define CHECK_RESULT(_result)                                                 \
{                                                                             \
    if(_result != COI_SUCCESS)                                                \
    {                                                                         \
        stringstream str;                                                     \
        str << "Error in " << __FILE__ << " at line " << __LINE__;            \
        str << " in function " << __FUNCTION__ << ": " <<                     \
                                    COIResultGetName(_result);                \
        cerr << str.str() << endl;                                            \
        exit(-1);                                                             \
    }                                                                         \
}

//Buffer that holds a block of characters and the last character from the
//previous buffer, which can be indexed with begin()cd [-1]
class Buffer
{
private:
    static const size_t _buffer_size = 100;
    char _storage[1+_buffer_size];  //Buffer size plus 1 character from
                                    //previous buffer
    size_t _size;

public:

    //[] operator for handling offset by 1 indices
    const char &operator[](const int &i) const
    {
        assert(-1 <= i && i < (int)_size);
        return(_storage[i+1]);              //Pointer to the ith character in
                                            //the buffer
    }

    //[] operator for handling offset by 1 indices
    char &operator[](const int &i)
    {
        assert(-1 <= i && i < (int)_size);
        return(_storage[i+1]);              //Pointer to the ith character in
                                            //the buffer
    }

    //Pointer to first character in the buffer
    char *begin()                   {return _storage+1;}
    //Pointer to first character in the buffer
    const char *begin() const       {return _storage+1;}
    //Pointer to one past last character in the buffer
    char *end()                     {return _storage+1+_size;}
    //Set the size of the buffer
    void set_size(size_t size)      {_size=size;}
    //Number of bytes appended to buffer.
    size_t size() const             {return _size;}
    //Number of  bytes a buffer can hold
    static size_t max_size()        {return _buffer_size;}

};

