/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */

//Project Includes
#include <intel-coi/common/COIMacros_common.h>
#include <intel-coi/sink/COIPipeline_sink.h>
#include <intel-coi/sink/COIProcess_sink.h>

#include "streaming_buffer_common.h"

// This functions steps through the buffer and tests each character and
// changes them to upper case when preceding character is a space
static void transform(Buffer &buffer)
{
    int prev_char_is_space = (buffer[-1] == ' ');
    for(char *s = buffer.begin(); s != buffer.end(); ++s)
    {
        if(prev_char_is_space && islower(*s))
        {
            *s = toupper(*s);
        }
        prev_char_is_space = isspace((unsigned char)*s);
    }
}

// This function performs the SINK side computation, in this case the SINK
// processes a character buffer
static int counter = 0;

COINATIVELIBEXPORT
void  SINKCompute(
            uint32_t         in_BufferCount,
            void**           in_ppBufferPointers,
            uint64_t*        in_pBufferLengths,
            void*            in_pMiscData,
            uint16_t         in_MiscDataLength,
            void*            in_pReturnValue,
            uint16_t         in_ReturnValueLength)
{
    UNREFERENCED_PARAM(in_BufferCount);
    UNREFERENCED_PARAM(in_ppBufferPointers);
    UNREFERENCED_PARAM(in_pBufferLengths);
    UNREFERENCED_PARAM(in_pMiscData);
    UNREFERENCED_PARAM(in_MiscDataLength);
    UNREFERENCED_PARAM(in_pReturnValue);
    UNREFERENCED_PARAM(in_ReturnValueLength);

    //Get pointers to the buffer data
    Buffer *inputBufferData  = (Buffer *)in_ppBufferPointers[0];
    Buffer *outputBufferData = (Buffer *)in_ppBufferPointers[1];

    //Copy the input buffer data into the output buffer
    *outputBufferData = *inputBufferData;

    //Transform the output buffer data
    transform(*outputBufferData);
    counter++;
    //print at every Xth iteration
    if(counter%XTH==0)
    {
        printf("Sink Compute finished\n");
    }
}

// main is automatically called whenever the source creates a process.
// In order for the source to create and use pipelines, the process must
// still exist.
// However, once main exits, the process that was created exits.
// Thus, main must live until there is no more work to be done.
int main(int , char**)
{
    printf("main called on the sink\n");
    fflush(stdout);

    // If you do not execute the function COIPipelineStartExecutingRunFunctions
    // then none of the other functions will be able to get called from the
    // source.
    // It is important to execute this only after you are done with
    // any setup that any of the run functions may need.
    COIRESULT err = COIPipelineStartExecutingRunFunctions();

    if( err != COI_SUCCESS )
    {
        return -1;
    }

    COIProcessWaitForShutdown();

    return 0;
}
